from typing import Any

from pydantic import BaseModel, Field, StrictStr, ConfigDict

from integra_bridge.dto.parameter import Parameter


class SkeletonConnector(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )

    type_connect: StrictStr = Field('', alias='typeConnect')
    style_form: dict[str, Any] = Field(default_factory=dict, alias='styleForm')
    relation_text: StrictStr = Field('', alias='relationText')
    parameters: list[Parameter] = Field(default_factory=list, alias='parameterList')



from typing import Annotated

from fastapi import APIRouter, Request, Query
from starlette import status
from integra_bridge.dto import Exchange
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.processor import Processor
from integra_bridge.services.processor import ProcessorHandler

processor_router = APIRouter(prefix="/processor", tags=["Процессоры"])


@processor_router.post(
    path='/execute',
    status_code=status.HTTP_200_OK,
    response_model=list[Exchange]
)
async def execute(
        request: Request,
        processor_title: Annotated[str, Query(alias='processorTitle')]
):
    exchange = await ProcessorHandler.execute(request=request, processor_title=processor_title)
    return [exchange]


@processor_router.post(
    path='/validate',
    status_code=status.HTTP_200_OK,
    response_model=ValidationResponse
)
async def validate(
        processor: Processor,
        title: Annotated[str, Query(alias='processorTitle')]
):
    return await ProcessorHandler.validate(processor, title=title)



from contextlib import asynccontextmanager
from typing import Callable
from pathlib import Path

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from integra_bridge.common.settings import SETTINGS
from integra_bridge.routers.v1.base import base_router
from integra_bridge.common.dependency_manager import dm


class Bridge:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(Bridge, cls).__new__(cls)
        return cls._instance

    def __init__(
            self,
            title: str = "Python-bridge",
            address: str = '',
            description: str = "Библиотека предназначена для подключения внешних Python сервисов",
            manual_path: Path = ''
    ):
        self.__title = title
        self.__address = address
        self.__description = description
        self.__manual_path = manual_path
        self.__handlers: list[Callable] = []

    def build(self):
        application = FastAPI(
            lifespan=self.lifespan,
            title=self.__title,
            description=self.__description,
            docs_url=f"{SETTINGS.API_PREFIX}/docs",
            redoc_url=f"{SETTINGS.API_PREFIX}/redoc",
            openapi_url=f"{SETTINGS.API_PREFIX}/openapi.json"
        )

        application.add_middleware(
            CORSMiddleware,
            allow_credentials=True,
            allow_origins=["*"],
            allow_methods=["*"],
            allow_headers=["*"],
        )
        application.include_router(router=base_router)
        # self.register_exception_handlers(application)

        return application

    def register_handlers(self, handlers: list[Callable]):
        self.__handlers = [handler for handler in handlers if handler not in self.__handlers]

    @asynccontextmanager
    async def lifespan(self, app: FastAPI):
        await self.on_startup()
        yield
        await self.on_shutdown()

    async def on_startup(self):
        await self.set_service_params()
        for handler in self.__handlers:
            handler()

    async def on_shutdown(self):
        ...

    async def set_service_params(self):
        dm.set_title(self.__title)
        dm.set_address(self.__address)
        dm.set_description(self.__description)
        dm.set_manual_path(self.__manual_path)

    @classmethod
    async def get_bridge(cls):
        return cls._instance



from abc import abstractmethod

from integra_bridge.adapters.BaseAdapter import BaseAdapter
from integra_bridge.dto import Exchange
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.processor import Processor


class ProcessorAdapter(BaseAdapter):
    def __init__(self):
        super().__init__()
        ProcessorAdapter.add_adapter(self)

    def __del__(self):
        ProcessorAdapter.remove_adapter(self)

    @abstractmethod
    async def execute(self, input_body: dict, params: dict) -> Exchange:
        ...

    async def validate(self, processor: Processor) -> ValidationResponse:
        return ValidationResponse(result=True)



import json
from abc import ABC
from typing import Any, NoReturn
import httpx

from integra_bridge.adapters.BaseAdapter import BaseAdapter
from integra_bridge.dto import Exchange, ConnectorToBlockView
from integra_bridge.dto.body import Body
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.connector import Connector
from integra_bridge.entity.output_status import OutputStatus


class ConnectorAdapter(BaseAdapter, ABC):
    _connector_to_block_views: dict[str, ConnectorToBlockView] = {}

    def __init__(self):
        super().__init__()
        ConnectorAdapter.add_adapter(self)

    def __del__(self):
        ConnectorAdapter.remove_adapter(self)

    async def pull_from_integra(self, input_body: dict, params: dict) -> OutputStatus:
        status = OutputStatus()
        return status

    @classmethod
    async def push_to_integra(
            cls,
            input_body: Any,
            connect_to_block_id: str = None
    ):
        try:
            string_body = json.dumps(input_body)
        except Exception as e:
            raise ValueError(f"input body is not json serializable: {str(e)}")
        if not connect_to_block_id:
            raise ValueError(f"connect to block id  is required")

        connector_to_block_view = ConnectorAdapter._connector_to_block_views.get(connect_to_block_id)
        if not connector_to_block_view:
            raise ValueError(f"connect to block view {connect_to_block_id} not found")

        input_body = Body(stringBody=string_body)
        exchange = Exchange(inputBody=input_body)
        exchange.block_id = connector_to_block_view.block_id
        exchange.input_connect_id = connector_to_block_view.connect_id
        exchange.company_id = connector_to_block_view.company_id

        url = f"{connector_to_block_view.url_integra_service}/api/external/connector/input/{connect_to_block_id}?connectorTitle={connector_to_block_view.connector_title}"
        return await cls.send(exchange, url)

    @classmethod
    async def send(cls, exchange: Exchange, url: str):
        auth = httpx.BasicAuth(username='admin', password='admin')
        async with httpx.AsyncClient(auth=auth, timeout=10) as client:
            response = await client.post(url, json=exchange.model_dump(by_alias=True), timeout=10)
            return response.status_code

    async def deploy_input_flow(self, exchange: Exchange, connector_title: str):
        connector_to_block_view = ConnectorToBlockView(
            connector_title=connector_title,
            company_id=exchange.company_id,
            block_id=exchange.block_id,
            connect_id=exchange.input_connect_id,
            url_integra_service=exchange.headers.get('urlIntegra'),
            exchange_deploy=exchange
        )
        connector_to_block_id = await connector_to_block_view.get_id()
        ConnectorAdapter._connector_to_block_views[connector_to_block_id] = connector_to_block_view
        print('!!!!!!!!DEPLOYED:', ConnectorAdapter._connector_to_block_views[connector_to_block_id].connector_title,
              ConnectorAdapter._connector_to_block_views.keys())
        await self.on_after_deploy(
            connection_id=connector_to_block_id,
            connector_params=connector_to_block_view.model_dump()
        )
        return exchange

    async def destroy_input_flow(self, exchange: Exchange, connector_title: str):
        connector_to_block_view = ConnectorToBlockView(
            connector_title=connector_title,
            company_id=exchange.company_id,
            block_id=exchange.block_id,
            connect_id=exchange.input_connect_id,
            exchange_deploy=exchange
        )
        connector_to_block_id = await connector_to_block_view.get_id()
        ConnectorAdapter._connector_to_block_views.pop(connector_to_block_id, None)
        print('!!!!!!!!DESTROYED:', ConnectorAdapter._connector_to_block_views.keys())
        await self.on_after_destroy(
            connection_id=connector_to_block_id,
            connector_params=connector_to_block_view.model_dump()
        )
        return exchange

    async def validate_output(self, connector: Connector) -> ValidationResponse:
        return ValidationResponse(result=True)

    async def validate_input(self, connector: Connector) -> ValidationResponse:
        return ValidationResponse(result=True)

    async def on_after_deploy(self, connection_id: str, connector_params: dict) -> NoReturn:
        ...

    async def on_after_destroy(self, connection_id: str, connector_params: dict) -> NoReturn:
        ...



from pydantic import BaseModel, StrictStr, Field

from integra_bridge.common.enums import DataSendingStatus


class OutputStatus(BaseModel):
    status: DataSendingStatus = Field(default=DataSendingStatus.success.value)
    error_message: StrictStr | None = None



from .connector import Connector
from .processor import Processor
from pydantic import BaseModel, StrictStr, Field, ConfigDict


class Block(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )

    id: StrictStr
    name: StrictStr | None = None
    description: StrictStr | None = None
    flow_id: StrictStr | None = Field(None, alias='flowId')
    company_id: StrictStr | None = Field(None, alias='companyId')
    flow_name: StrictStr | None = Field(None, alias='flowName')
    is_active: bool | None = Field(None, alias='isActive')
    is_guaranteed_order: bool | None = Field(None, alias='isGuaranteedOrder')
    is_guaranteed_delivery: bool | None = Field(None, alias='isGuaranteedDelivery')
    is_active_guaranteed_delivery: bool | None = Field(None, alias='isActiveGuaranteedDelivery')
    is_limited_count_delivery: bool | None = Field(None, alias='isLimitedCountDelivery')
    limited_count: int | None = Field(None, alias='limitedCount')
    input_list: list[Connector] | None = Field(None, alias='inputList')
    enriched_inputs: list[Connector] | None = Field(None, alias='enrichedInputList')
    processors: list[Processor] | None = Field(None, alias='processorList')
    enriched_processors: list[Processor] | None = Field(None, alias='enrichedProcessorList')
    outputs: list[Connector] | None = Field(None, alias='outputList')
    enriched_outputs: list[Connector] | None = Field(None, alias='enrichedOutputList')



import json

from pydantic import BaseModel, Field, StrictStr, ConfigDict
from typing import Any
from pydantic.json import pydantic_encoder


class Body(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        json_encoders={dict: pydantic_encoder}
    )
    packet_id: StrictStr | None = Field(None, alias='packetId')
    type: StrictStr | None = None
    string_body: StrictStr = Field(alias='stringBody')
    object_string: StrictStr | None = Field(None, alias='objectString', exclude=True)
    object: Any | None = Field(None, exclude=True)
    output_body: StrictStr | None = Field(None, alias='outputBody')
    output_type: StrictStr | None = Field(None, alias='outputType')

    def __init__(self, **data):
        super().__init__(**data)
        self.object = self.parse_object()

    def parse_object(self):
        if not self.string_body or not self.string_body.strip():
            self.string_body = "{}"
        try:
            obj = json.loads(self.string_body)
            self.type = "json"
        except json.JSONDecodeError:
            obj = self.string_body
            self.type = "string"
        self.object_string = self.string_body
        return obj

    def get_type(self):
        self.parse_object()
        return self.type

    def set_string_body(self, string_body: str):
        self.string_body = string_body
        self.object = None
        self.object_string = None

    def get_output_body(self):
        try:
            obj = self.parse_object()
            return json.dumps(obj, default=pydantic_encoder)
        except Exception:
            return self.string_body

    def __str__(self):
        return (f"Body(packet_id={self.packet_id}, type={self.type}, string_body={self.string_body}, "
                f"object_string={self.object_string}, object={self.object}, output_body={self.output_body}, "
                f"output_type={self.output_type})")



from fastapi import HTTPException, Request
from orjson import orjson
from pydantic import ValidationError
from starlette import status

from integra_bridge.dto import Exchange
from integra_bridge.adapters import ConnectorAdapter
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.connector import Connector


class ConnectorHandler:
    @classmethod
    async def execute_output_exchange(cls, request: Request, title: str):
        connector_adapter = await cls.get_connector_by_title(title)
        try:
            exchange = await request.json()
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Output body is not JSON serializable")
        output_body = exchange.get("outputBody", {}).get("stringBody", {})
        try:
            inner_body = orjson.loads(output_body)
        except ValueError:
            inner_body = output_body
        params = exchange.get("outputConnect", {}).get("params", {})
        inner_params = params
        try:
            output_status = await connector_adapter.pull_from_integra(inner_body, inner_params)
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=f'Connector error: {str(e)}')
        if output_status.error_message:
            exchange['exception'] = output_status.error_message
        return exchange

    @classmethod
    async def deploy_input(cls, request: Request, title: str) -> Exchange:
        exchange = await cls.get_exchange_object(request)
        connector = await ConnectorHandler.get_connector_by_title(title)
        await connector.deploy_input_flow(exchange=exchange, connector_title=title)
        return exchange

    @classmethod
    async def destroy_input(cls, request: Request, title: str) -> Exchange:
        exchange = await cls.get_exchange_object(request)
        connector = await ConnectorHandler.get_connector_by_title(title)
        await connector.destroy_input_flow(exchange=exchange, connector_title=title)
        return exchange

    @classmethod
    async def validate_output(cls, connector: Connector, title: str) -> ValidationResponse:
        try:
            connector_adapter = await cls.get_connector_by_title(title)
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
        return await connector_adapter.validate_output(connector)

    @classmethod
    async def validate_input(cls, connector: Connector, title: str) -> ValidationResponse:
        try:
            connector_adapter = await cls.get_connector_by_title(title)
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
        return await connector_adapter.validate_input(connector)

    @staticmethod
    async def get_exchange_object(request: Request) -> Exchange:
        try:
            exchange_dict = await request.json()
            exchange = Exchange(**exchange_dict)
        except ValidationError as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=f"Exchange structure is unvalid: {e}")
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                                detail=f"Input body is not JSON serializable: {e}")
        return exchange

    @staticmethod
    async def get_connector_by_title(title: str) -> ConnectorAdapter:
        for connector_adapter in ConnectorAdapter.get_adapters():
            connector_view = await connector_adapter.get_view()
            if connector_view.title.lower() == title.lower():
                return connector_adapter
        raise HTTPException(status_code=404, detail=f'Connector not found: {title}')



from integra_bridge.dto.parameter import Parameter
from integra_bridge.entity.processor import Processor
from .output_status import OutputStatus

__all__ = (
    'Parameter',
    'Processor',
    'OutputStatus',
)



from pathlib import Path

from pydantic import PositiveInt, StrictStr
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        case_sensitive=True,
        env_file=Path(__file__).parent.parent.parent / ".env",
        env_file_encoding="utf-8",
        extra="allow",
    )
    DEFAULT_TIMEOUT: PositiveInt = 60
    CASH_LIFETIME: PositiveInt = 1000
    CASH_SIZE: PositiveInt = 1024
    API_PREFIX: StrictStr = "/api/integra"
    BACKGROUND_REFRESH_SECONDS: PositiveInt = 60
    MAX_BACKGROUND_REFRESH_ATTEMPTS: PositiveInt = 3


SETTINGS = Settings()



import uuid
from pydantic import BaseModel, UUID4, Field, StrictStr, ConfigDict
from integra_bridge.dto.skeletons.connector import SkeletonConnector


class ConnectorView(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    connector_id: UUID4 = Field(default_factory=uuid.uuid4, alias='connectorID')
    title: StrictStr = Field(alias='connectorTitle')
    input_description: StrictStr = Field('', alias='inputConnectorDescription')
    output_description: StrictStr = Field('', alias='outputConnectorDescription')
    skeleton_input_connect: SkeletonConnector | None = Field(None, alias='skeletonInputConnector')
    skeleton_output_connect: SkeletonConnector | None = Field(None, alias='skeletonOutputConnector')



from pydantic import BaseModel, ConfigDict, StrictStr, Field
from typing import Any
from pydantic.json import pydantic_encoder


class ConnectorToBlockView(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        json_encoders={dict: pydantic_encoder}
    )
    connector_title: StrictStr = Field(None, alias='connectorTitle')
    company_id: StrictStr = Field(None, alias='companyId')
    block_id: StrictStr = Field(None, alias='blockId')
    connect_id: StrictStr = Field(None, alias='connectId')
    url_integra_service: StrictStr | None = Field(None, alias='urlIntegraService')
    exchange_deploy: Any = Field(None, alias='exchangeDeploy')

    async def get_id(self):
        return f'{self.company_id}_{self.block_id}_{self.connect_id}'



from abc import ABC, abstractmethod


class ExternalService(ABC):
    _instances = set()

    def __init__(self):
        ExternalService.add_service(self)

    @abstractmethod
    async def execute(self):
        pass

    @classmethod
    async def get_instances(cls):
        return [instance for instance in cls._instances if isinstance(instance, cls)]

    @classmethod
    def add_service(cls, service):
        cls._instances.add(service)

    @classmethod
    def remove_service(cls, service):
        cls._instances.remove(service)



from pydantic import BaseModel, StrictStr, Field, ConfigDict
from integra_bridge.dto import SkeletonProcessor


class ProcessorView(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    title: StrictStr | None = Field("", alias='processorTitle')
    description: StrictStr | None = Field("", alias='processorDescription')
    skeleton: SkeletonProcessor = Field("", alias='skeletonProcessor')



from pydantic import BaseModel, StrictStr, Field, ConfigDict
from typing import Any


class Processor(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )

    id: StrictStr
    name: StrictStr | None = None
    description: StrictStr | None = None
    number: int | None = None
    type_processor: StrictStr | None = Field(None, alias='typeProcessor')
    params: dict[StrictStr, StrictStr] | None = None
    params_object: dict[StrictStr, Any] | None = Field(None, alias='paramsObject')



from pydantic import BaseModel, StrictStr, Field, ConfigDict
from datetime import datetime

from integra_bridge.dto import ConnectorView
from integra_bridge.dto import ProcessorView


class ExternalServiceConfigResponse(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    service_name: StrictStr = Field('', alias='serviceName')
    service_address: StrictStr = Field('', alias='serviceAddress')
    application_start_date: datetime | None = Field(None, alias='applicationStartDate')
    connector_views: list[ConnectorView] | None = Field(default_factory=list, alias='connectorViewList')
    processor_views: list[ProcessorView] | None = Field(default_factory=list, alias='processorViewList')
    manual_file_name: StrictStr | None = Field(None, alias='manualFileName')



from abc import ABC, abstractmethod


class BaseAdapter(ABC):
    _adapters: dict[str, set] = dict()

    def __init__(self):
        self.__view = None

    @abstractmethod
    async def get_view(self):
        ...

    @property
    def view(self):
        return self.__view

    @classmethod
    def get_adapters(cls) -> list:
        return cls._adapters.get(cls.__name__, [])

    @classmethod
    def add_adapter(cls, service):
        group = cls._adapters.get(cls.__name__, None)
        if group:
            group.add(service)
        else:
            cls._adapters[cls.__name__] = {service, }

    @classmethod
    def remove_adapter(cls, service):
        group = cls._adapters.get(cls.__name__, None)
        if group:
            group.remove(service)



from pydantic import BaseModel, Field, StrictStr, ConfigDict


class ValidationResponse(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    result: bool = True
    # Блокирующие ошибки
    params_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict, alias='paramsDescription')
    query_params_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict,
                                                                       alias='queryParamsDescription'
                                                                       )
    filter_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict, alias='filterDescription')

    # Неблокирующие
    warning_params_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict,
                                                                         alias='warningParamsDescription')
    warning_query_params_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict,
                                                                               alias='warningQueryParamsDescription')
    warning_filter_description: dict[StrictStr, list[StrictStr]] = Field(default_factory=dict,
                                                                         alias='warningFilterDescription')



from fastapi import APIRouter

from integra_bridge.common.settings import SETTINGS
from integra_bridge.routers.v1.configuration import configuration_router
from integra_bridge.routers.v1.connector import connector_router
from integra_bridge.routers.v1.processor import processor_router

base_router = APIRouter(prefix=SETTINGS.API_PREFIX)

base_router.include_router(configuration_router)
base_router.include_router(processor_router)
base_router.include_router(connector_router)



import json
from orjson import orjson
from pydantic import ValidationError

from integra_bridge.adapters.ProcessorAdapter import ProcessorAdapter
from fastapi import HTTPException, Request
from starlette import status

from integra_bridge.dto import Exchange
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.processor import Processor


class ProcessorHandler:
    @classmethod
    async def execute(cls, request: Request, processor_title: str) -> Exchange:
        processor_adapter = await cls.get_processor_by_title(processor_title)

        try:
            exchange = await request.json()
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Input body is not JSON serializable")
        input_body = exchange.get("body", {}).get("stringBody", {})
        try:
            inner_body = orjson.loads(input_body)
        except ValueError:
            inner_body = input_body

        params = exchange.get("processor", {}).get("params", {})
        try:
            updated_body = await processor_adapter.execute(inner_body, params)
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_500_INTERNAL_SERVER_ERROR, detail=f'Processor error: {str(e)}')
        try:
            updated_body = json.dumps(updated_body, ensure_ascii=False)
            body_type = "json"
        except ValueError:
            body_type = "string"
        exchange["body"]["type"] = body_type
        exchange["body"]["stringBody"] = updated_body
        try:
            output_exchange = Exchange(**exchange)
        except ValidationError as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
        return output_exchange

    @classmethod
    async def validate(cls, processor: Processor, title: str) -> ValidationResponse:
        try:
            processor_adapter = await cls.get_processor_by_title(title)
        except Exception as e:
            raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail=str(e))
        return await processor_adapter.validate(processor)

    @classmethod
    async def get_processor_by_title(cls, title: str) -> ProcessorAdapter:
        for processor_adapter in ProcessorAdapter.get_adapters():
            processor_view = await processor_adapter.get_view()
            if processor_view.title.lower() == title.lower():
                return processor_adapter
        raise HTTPException(status_code=404, detail=f'Processor not found: {title} ')



from enum import Enum
from typing import Any

from pydantic import BaseModel, StrictStr, Field, ConfigDict

from integra_bridge.common.enums import ParameterType


class Parameter(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    name: StrictStr
    label: StrictStr | None = Field(default=None)
    type: Enum = Field(default=ParameterType.TextField)
    # Стиль элемента, где рендерятся внутренние элементы
    children_box_style: dict[str, Any] | None = Field(default=None, alias='childrenBoxStyle')
    # Стиль элемента
    style: dict[str, Any] | None = Field(default=None)
    # Стиль контейнера, где находится элемент
    box_style: dict[str, Any] | None = Field(default=None, alias='boxStyle')
    props: dict[str, Any] | None = Field(default=None)

    description: StrictStr | None = Field(default=None)
    default_value: StrictStr | None = Field(default=None, alias='defaultValue')
    # Для настроек, текущее значение
    value: StrictStr | None = Field(default=None)

    # list of Parameter
    parameters: list['Parameter'] | None = Field(default=None, alias='parameterList')

    default_option: StrictStr | None = Field(default=None, alias='defaultOption')
    options: list[str] | None = Field(default=None)
    object_options: list[Any] | None = Field(default=None, alias='objectOptions')

    bottom_text: StrictStr | None = Field(default=None, alias='bottomText')
    is_add_value: bool = Field(..., alias='isAddValue')

    is_change: bool = Field(default=True, alias='isChange')
    is_required: bool = Field(default=False, alias='isRequired')
    is_visible: bool = Field(default=True, alias='isVisible')
    is_disabled: bool = Field(default=False, alias='isDisabled')
    is_external: bool = Field(default=False, alias='isExternal')


Parameter.update_forward_refs()




from enum import Enum


class ParameterType(Enum):
    TextField = "TextField"
    Cron = "Cron"
    Select = "Select"
    Autocomplete = "Autocomplete"
    CheckBox = "CheckBox"
    Switch = "Switch"
    SwitchWithParameter = "SwitchWithParameter"
    Code = "Code"
    FilterList = "FilterList"
    SelectByParameter = "SelectByParameter"
    Password = "Password"
    ListObject = "ListObject"
    ListString = "ListString"
    Group = "Group"


class ConnectorDirection(Enum):
    input = "input"
    output = "output"
    both = "both"


class DataSendingStatus(Enum):
    success = "success"
    failed = "failed"



from dataclasses import dataclass
from datetime import datetime, UTC
from pathlib import Path

from typing import NoReturn


@dataclass
class DependencyManager:
    def __init__(self):
        self.__title = ''
        self.__address = ''
        self.__description = ''
        self.__manual_path = ''
        self.__application_start_date = datetime.now(UTC)

    def set_title(self, title: str) -> NoReturn:
        self.__title = title

    def set_description(self, description: str) -> NoReturn:
        self.__description = description

    def set_manual_path(self, manual_path: Path) -> NoReturn:
        self.__manual_path = manual_path

    def set_address(self, address: str) -> NoReturn:
        self.__address = address

    @property
    def application_start_date(self) -> datetime:
        return self.__application_start_date

    @property
    def title(self) -> str:
        return self.__title

    @property
    def description(self) -> str:
        return self.__description

    @property
    def address(self) -> str:
        return self.__address

    @property
    def manual_path(self) -> Path:
        return self.__manual_path


dm = DependencyManager()



from .skeletons.processor import SkeletonProcessor
from .skeletons.connector import SkeletonConnector
from .views.processor_view import ProcessorView
from .views.connector_view import ConnectorView
from .exchange import Exchange
from .connector_to_block_view import ConnectorToBlockView


__all__ = (
    'SkeletonProcessor',
    'SkeletonConnector',
    'ProcessorView',
    'ConnectorView',
    'Exchange',
    'ConnectorToBlockView',
)



from typing import Annotated

from fastapi import APIRouter, Request, Query
from starlette import status

from integra_bridge.dto import Exchange
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.connector import Connector
from integra_bridge.services.connector import ConnectorHandler

connector_router = APIRouter(prefix="/connector", tags=["Коннекторы"])


@connector_router.post(
    path='/output',
    status_code=status.HTTP_200_OK,
    response_model_by_alias=True
)
async def output(
        request: Request,
        title: Annotated[str, Query(alias='connectorTitle')]
):
    return await ConnectorHandler.execute_output_exchange(request=request, title=title)


@connector_router.post(
    path='/validateOutput',
    status_code=status.HTTP_200_OK,
    response_model=ValidationResponse
)
async def validate_output(
        connector: Connector,
        title: Annotated[str, Query(alias='connectorTitle')]
):
    return await ConnectorHandler.validate_output(connector, title=title)


@connector_router.post(
    path='/validateInput',
    status_code=status.HTTP_200_OK,
    response_model=ValidationResponse
)
async def validate_output(
        connector: Connector,
        title: Annotated[str, Query(alias='connectorTitle')]
):
    return await ConnectorHandler.validate_input(connector, title=title)


@connector_router.post(
    path='/deployInput',
    status_code=status.HTTP_200_OK,
    response_model_by_alias=True,
    response_model=Exchange
)
async def deploy_input(
        request: Request,
        title: Annotated[str, Query(alias='connectorTitle')]
):
    return await ConnectorHandler.deploy_input(request=request, title=title)


@connector_router.post(
    path='/destroyInput',
    status_code=status.HTTP_200_OK,
    response_model_by_alias=True,
    response_model=Exchange
)
async def destroy_input(
        request: Request,
        title: Annotated[str, Query(alias='connectorTitle')]
):
    return await ConnectorHandler.destroy_input(request=request, title=title)



from pydantic import BaseModel, Field, StrictStr, ConfigDict
from typing import Any

from .body import Body
from integra_bridge.entity.block import Block
from integra_bridge.entity.connector import Connector
from integra_bridge.entity.processor import Processor


class Exchange(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    id_executor_history: StrictStr | None = Field(None, alias='idExecutorHistory')
    id: StrictStr | None = None
    company_id: StrictStr | None = Field(None, alias='companyId')
    input_headers: dict[StrictStr, StrictStr] | None = Field(default_factory=dict, alias='inputHeaders')
    headers: dict[StrictStr, StrictStr] | None = {}
    output_headers: dict[StrictStr, StrictStr] | None = Field(default_factory=dict, alias='outputHeaders')
    input_path_params: dict[StrictStr, StrictStr] | None = Field(default_factory=dict, alias='inputPathParams')
    input_query_params: dict[StrictStr, StrictStr] | None = Field(default_factory=dict, alias='inputQueryParams')
    input_body: Body | None = Field(None, alias='inputBody')
    body: Body | None = None
    output_body: Body | None = Field(None, alias='outputBody')

    # Exchange
    exchange: Any | None = None

    block_id: StrictStr | None = Field(None, alias='blockId')
    block: Block | None = None
    input_connect_id: StrictStr | None = Field(None, alias='inputConnectId')
    input_connect: Connector | None = Field(None, alias='inputConnect')
    type_input_connect: StrictStr | None = Field(None, alias='typeInputConnect')

    # Если true то будет принудительно передеплоин
    is_forcible: bool | None = Field(None, alias='isForcible')

    # Если true то принудительно будет отработан, даже если блок не активен
    is_execute_block_forcible: bool | None = Field(None, alias='isExecuteBlockForcible')

    # Если null == все процессоры используем
    use_processor_ids: list[StrictStr] | None = Field(None, alias='useProcessorIdList')

    processor_id: StrictStr | None = Field(None, alias='processorId')
    processor: Processor | None = None
    is_skip: bool | None = Field(None, alias='isSkip')

    # Если false = то отфильтрофан, далее не идет.
    is_filtered: bool | None = Field(None, alias='isFiltered')

    # Если null - то отправляем.
    is_send_output_connect: bool | None = Field(None, alias='isSendOutputConnect')

    # Если true - то выполняется интеграционный тест
    is_integration_test: bool | None = Field(None, alias='isIntegrationTest')

    output_connect_id: StrictStr | None = Field(None, alias='outputConnectId')
    output_connect: Connector | None = Field(None, alias='outputConnect')
    input_object_map: dict[StrictStr, Any] | None = Field(default_factory=dict, alias='inputObjectMap')
    object_map: dict[StrictStr, Any] | None = Field(default_factory=dict, alias='objectMap')
    output_object_map: dict[StrictStr, Any] | None = Field(default_factory=dict, alias='outputObjectMap')

    # Обработка ошибок,логика по переотправке
    error_packet_id: StrictStr | None = Field(None, alias='errorPacketId')

    # Если true, то запущен из ErrorPacketService
    is_run_error_packet: bool | None = Field(None, alias='isRunErrorPacket')

    is_finish: bool | None = Field(None, alias='isFinish')
    is_exist_error_packet: bool | None = Field(None, alias='isExistErrorPacket')
    is_guaranteed_order: bool | None = Field(None, alias='isGuaranteedOrder')
    is_guaranteed_delivery: bool | None = Field(None, alias='isGuaranteedDelivery')
    is_limited_count_delivery: bool | None = Field(None, alias='isLimitedCountDelivery')
    limited_count: int | None = Field(None, alias='limitedCount')
    processor_id_exception: StrictStr = Field(None, alias='processorIdException')
    status: StrictStr | None = None
    exception: StrictStr | None = None



from pydantic import BaseModel, Field, ConfigDict
from integra_bridge.dto.parameter import Parameter


class SkeletonProcessor(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )
    type_processor: str = Field("", alias='typeProcessor')
    parameters: list[Parameter] = Field(default_factory=list, alias='parameterList')



from pydantic import BaseModel, Field, StrictStr, ConfigDict
from typing import Any


class Connector(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
    )

    id: StrictStr
    company_id: StrictStr | None = Field(None, alias='companyId')
    name: StrictStr | None = None
    description: StrictStr | None = None
    number: int | None = None
    type_connect: StrictStr | None = Field(None, alias='typeConnect')
    input_type_format: StrictStr | None = Field(None, alias='inputTypeFormat')
    output_type_format: StrictStr | None = Field(None, alias='outputTypeFormat')
    params: dict[StrictStr, StrictStr] | None = None
    query_params: dict[StrictStr, StrictStr] | None = Field(None, alias='queryParams')
    #  массивы, объекты
    params_object: dict[StrictStr, Any] | None = Field(None, alias='paramsObject')



from integra_bridge.bridge import Bridge
from integra_bridge.common.enums import ParameterType, DataSendingStatus
from integra_bridge.dto.responces.validation import ValidationResponse

__all__ = (
    'Bridge',
    'ParameterType',
    'DataSendingStatus',
    'ValidationResponse',
)



from fastapi.responses import FileResponse

from integra_bridge.adapters import ProcessorAdapter, ConnectorAdapter
from integra_bridge.api_router import APIRouter
from integra_bridge.dto.responces.external_service import ExternalServiceConfigResponse
from integra_bridge.common.dependency_manager import dm

configuration_router = APIRouter(prefix='/configuration', tags=["Работа с конфигурациями внешних сервисов"])


@configuration_router.get(
    path='/',
    response_model=ExternalServiceConfigResponse,
    response_model_exclude_none=True,
    response_model_by_alias=True
)
async def get_configurations():
    processors = ProcessorAdapter.get_adapters()
    processor_views = []
    for processor in processors:
        processor_view = await processor.get_view()
        processor_views.append(processor_view)

    connectors = ConnectorAdapter.get_adapters()
    connector_views = []
    for connector in connectors:
        connector_view = await connector.get_view()
        connector_views.append(connector_view)

    response = ExternalServiceConfigResponse(
        service_name=dm.title,
        service_address=dm.address,
        application_start_date=dm.application_start_date,
        processor_views=processor_views,
        connector_views=connector_views,
        manual_file_name=dm.manual_path.name
    )
    return response


@configuration_router.get(
    path='/manual'
)
async def get_configurations():
    manual_path = dm.manual_path
    if manual_path.exists():
        return FileResponse(manual_path, media_type='application/octet-stream', filename=dm.manual_path.name)
    else:
        return {"error": "File not found"}
